﻿using System;
using FluentAssertions;
using Website.Models.Git;
using Website.ViewModels;
using Xunit;

namespace Website.Tests.VIewModels {
	public class GitCommitViewModelTests {
		[Fact]
		public void Construct__WithRepositoryAndBranchAndCommit_SetsUpProperties() {
			var repo = new Repository {
				Url = "repo url",
				Name = "repo name"
			};

			var branch = new Branch {
				Name = "branch name"
			};

			var commit = new Commit {
				Id = "id",
				Message = "message",
				Timestamp = new DateTimeOffset(2020, 04, 13, 1, 2, 3, TimeSpan.Zero),
				Url = "commit url"
			};

			var model = new GitCommitViewModel(repo, branch, commit);

			model.RepositoryName.Should().Be("repo name");
			model.RepositoryUrl.Should().Be("repo url");
			model.BranchName.Should().Be("branch name");
			model.Hash.Should().Be("id");
			model.CommitMessage.Should().Be("message");
			model.CommitUrl.Should().Be("commit url");
			model.Timestamp.Should().Be("01:02:03 on Monday the 13th of April 2020");
		}
	}
}