using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Caching.Memory;
using System.Threading.Tasks;
using Website.Data;
using Website.Models.Mailboxes;

namespace Website.Controllers {
	[Authorize]
	public class Admin : Controller {
		private readonly IApiKeyManager _apiKeyManager;
		private readonly IMailboxesApi _mailboxesApi;
		private readonly IMemoryCache _cache;

		public Admin(IApiKeyManager apiKeyManager, IMailboxesApi mailboxesApi, IMemoryCache cache) {
			_apiKeyManager = apiKeyManager;
			_mailboxesApi = mailboxesApi;
			_cache = cache;
		}

		public async Task<IActionResult> ApiKeys() {
			var keys = await _apiKeyManager.List();
			return View(keys);
		}

		public async Task<IActionResult> Mailboxes() {
			var aliases = await _mailboxesApi.ListAliases();
			return View(aliases);
		}

		public async Task<IActionResult> CreateApiKey(string name) {
			await _apiKeyManager.Create(name);
			return RedirectToAction(nameof(ApiKeys));
		}

		public async Task<IActionResult> EnableApiKey(string key) {
			await _apiKeyManager.Enable(key);
			return RedirectToAction(nameof(ApiKeys));
		}

		public async Task<IActionResult> DisableApiKey(string key) {
			await _apiKeyManager.Disable(key);
			return RedirectToAction(nameof(ApiKeys));
		}

		public async Task<IActionResult> DeleteApiKey(string key) {
			await _apiKeyManager.Delete(key);
			return RedirectToAction(nameof(ApiKeys));
		}

		public async Task<IActionResult> EnableAlias(int id) {
			await _mailboxesApi.EnableAlias(id);
			return RedirectToAction(nameof(Mailboxes));
		}

		public async Task<IActionResult> DisableAlias(int id) {
			await _mailboxesApi.DisableAlias(id);
			return RedirectToAction(nameof(Mailboxes));
		}

		public async Task<IActionResult> DeleteAlias(int id) {
			await _mailboxesApi.DeleteAlias(id);
			return RedirectToAction(nameof(Mailboxes));
		}

		public async Task<IActionResult> CreateAlias(string source, string destination, bool active) {
			var alias = new NewAlias {Source = source, Destination = destination, Active = active};
			await _mailboxesApi.CreateAlias(alias);
			return RedirectToAction(nameof(Mailboxes));
		}

		[HttpGet]
		public async Task<IActionResult> Cache() => View(_cache);

		[HttpPost]
		public async Task<IActionResult> Cache(bool reset) {
			if (reset && _cache is MemoryCache cache) 
				cache.Compact(1.0);
			return RedirectToAction(nameof(Cache));
		}
	}
}